@@ -7,5 +7,5 @@
 void __naked barebox_arm_reset_vector(void)
 {
 	arm_cpu_lowlevel_init();
-	barebox_arm_entry(S3C_SDRAM_BASE, SZ_128M, 0);
+	barebox_arm_entry(S3C_SDRAM_BASE, SZ_128M, NULL);
 }
