@@ -16,17 +16,6 @@
 #ifndef __ASM_MACH_GPIO_H
 #define __ASM_MACH_GPIO_H
 
-#include <types.h>
-
-#if defined CONFIG_ARCH_IMX23
-# include <mach/iomux-imx23.h>
-#endif
-#if defined CONFIG_ARCH_IMX28
-# include <mach/iomux-imx28.h>
-#endif
-
 #include <asm-generic/gpio.h>
 
-void imx_gpio_mode(uint32_t);
-
 #endif /* __ASM_MACH_GPIO_H */
