@@ -6,6 +6,10 @@ const char version_string[] =
 	"barebox " UTS_RELEASE " " UTS_VERSION "\n";
 EXPORT_SYMBOL(version_string);
 
+const char release_string[] =
+	"barebox-" UTS_RELEASE;
+EXPORT_SYMBOL(release_string);
+
 void barebox_banner (void)
 {
 	pr_info("\n\n%s\n\n", version_string);
