@@ -402,7 +402,7 @@ static int tse_eth_rx(struct eth_device *edev)
 		ALT_SGDMA_DESCRIPTOR_STATUS_TERMINATED_BY_EOP_MSK) {
 
 		packet_length = rx_desc->actual_bytes_transferred;
-		net_receive(NetRxPackets[0], packet_length);
+		net_receive(edev, NetRxPackets[0], packet_length);
 
 		/* Clear Run */
 		rx_sgdma->control = (rx_sgdma->control & (~ALT_SGDMA_CONTROL_RUN_MSK));
