@@ -1,8 +1,6 @@
 #ifndef __INCLUDE_SPI_H
 #define __INCLUDE_SPI_H
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
-
 #include <driver.h>
 #include <linux/string.h>
 
@@ -431,8 +429,6 @@ static inline ssize_t spi_w8r8(struct spi_device *spi, u8 cmd)
 	return (status < 0) ? status : result;
 }
 
-#endif /* DOXYGEN_SHOULD_SKIP_THIS */
-
 extern struct bus_type spi_bus;
 
 struct spi_master *spi_get_master(int bus);
